@@ -11,27 +11,29 @@ This module only works with Python 2.0 since all string parameters
 are assumed to be Unicode objects and string methods are used instead
 string module.
 
-$Id: forms.py,v 1.20 2002/01/27 16:13:58 michael Exp $
+$Id: forms.py,v 1.35 2003/12/23 15:11:19 michael Exp $
 """ 
 
-__version__ = '0.14.1'
+__version__ = '0.15.0'
 
-import sys,os,types,re,urllib
+import sys,types,re,urllib
 import helper
 
-_ESCAPE_HTML_CHARS=list('&<>":={}()')
-_ESCAPE_HTML_CHARS_TRANS = [
-  (c,'&#%d;' % ord(c))
-  for c in _ESCAPE_HTML_CHARS
-]
-
 def escapeHTML(s):
   """
   Escape all characters with a special meaning in HTML
   to appropriate character tags
   """
-  for c,e in _ESCAPE_HTML_CHARS_TRANS:
-    s = s.replace(c,e)
+  s = s.replace('&','&#38;')
+  s = s.replace('<','&#60;')
+  s = s.replace('>','&#62;')
+  s = s.replace('"','&#34;')
+  s = s.replace(':','&#58;')
+  s = s.replace('=','&#61;')
+  s = s.replace('{','&#123;')
+  s = s.replace('}','&#125;')
+  s = s.replace('(','&#40;')
+  s = s.replace(')','&#41;')
   return s
 
 
@@ -86,10 +88,15 @@ class Field:
     else:
       return ''
 
-  def labelHTML(self,labelText=None):
-    if labelText is None:
-      labelText = self.text
-    return '<label for="%s">%s</label>' % (self.name,labelText)
+  def idAttrStr(self,id_value):
+    if id_value is None:
+      return ''
+    else:
+      return 'id="%s" ' % (id_value)
+
+  def labelHTML(self,labelText=None,for_value=None):
+    labelText = (labelText or self.text).encode(self.charset)
+    return '<label for="%s">%s</label>' % (for_value or self.name,labelText)
 
   def getValue(self):
     """
@@ -108,13 +115,10 @@ class Field:
     Mainly this is used by the application if self.default shall
     be changed after initializing the field object.
     """
-    if default is None:
-      self.default=''
+    if type(default)==types.ListType:
+      self.default = [i for i in default if i!=None]
     else:
-      if type(default)==types.ListType:
-        self.default = [i for i in default if i!=None]
-      else:
-        self.default = default
+      self.default = default or ''
 
   def _patternAndOptions(self,pattern):
     """
@@ -124,9 +128,7 @@ class Field:
         Either a string containing a regex pattern,
         a tuple (pattern string, pattern options) or None.
     """
-    if pattern is None:
-      return None,0
-    elif type(pattern) is types.TupleType:
+    if type(pattern) is types.TupleType:
       return pattern
     else:
       return pattern, 0
@@ -210,11 +212,8 @@ class Field:
     self.charset = charset
 
   def _defaultValue(self,default):
-    """HTML output of default."""
-    if default is None:
-      return self.__dict__.get('default','')
-    else:
-      return default
+    """returns default value"""
+    return default or self.__dict__.get('default','')
 
   def _defaultHTML(self,default):
     """HTML output of default."""
@@ -257,11 +256,12 @@ class Textarea(Field):
     patternoptions = patternoptions | re.M|re.S
     Field.setRegex(self,(patternstring,patternoptions))
 
-  def inputHTML(self,default=None):
+  def inputHTML(self,default=None,id_value=None,title=None):
     """Returns string with HTML input field."""
     return self.inputHTMLTemplate % (
-      '<textarea id="%s" name="%s" %s rows="%d" cols="%d">%s</textarea>' % (
-        self.name,
+      '<textarea %stitle="%s" name="%s" %s rows="%d" cols="%d">%s</textarea>' % (
+        self.idAttrStr(id_value),
+        (title or self.text).encode(self.charset),
 	self.name,
 	self._accessKeyAttr(),
 	self.rows,self.cols,
@@ -290,18 +290,21 @@ class Input(Field):
     self,name,text,maxLen,maxValues,pattern,required=0,default=None,accessKey='',
     size=None
   ):
-    if size is None:
-      self.size = maxLen
-    else:
-      self.size = size
+    self.size = size or maxLen
     Field.__init__(
       self,name,text,maxLen,maxValues,pattern,required,default,accessKey,
     )
 
-  def inputHTML(self,default=None):
+  def inputHTML(self,default=None,id_value=None,title=None):
     return self.inputHTMLTemplate % (
-	'<input id="%s" name="%s" %s maxlength="%d" size="%d" value="%s">' % (
-	self.name,self.name,self._accessKeyAttr(),self.maxLen,self.size,self._defaultHTML(default)
+	'<input %stitle="%s" name="%s" %s maxlength="%d" size="%d" value="%s">' % (
+	self.idAttrStr(id_value),
+        (title or self.text).encode(self.charset),
+        self.name,
+        self._accessKeyAttr(),
+        self.maxLen,
+        self.size,
+        self._defaultHTML(default)
       )
     )
 
@@ -313,23 +316,58 @@ class HiddenInput(Input):
   """
 
   def __init__(
-    self,name,text,maxLen,maxValues,pattern,required=0,default=None,
-    show=0
+    self,name,text,maxLen,maxValues,pattern,required=0,default=None,accessKey=''
   ):
     Input.__init__(
-      self,name,text,maxLen,maxValues,pattern,required,default
+      self,name,text,maxLen,maxValues,pattern,required,default,accessKey,
     )
-    self.show = show
 
-  def inputHTML(self,default=None):
+  def inputHTML(self,default=None,id_value=None,title=None,show=0):
     default_html = self._defaultHTML(default)
-    if self.show:
+    if show:
       default_str = default_html
     else:
       default_str = ''
     return self.inputHTMLTemplate % (
-	'<input type="hidden" id="%s" name="%s" %s  value="%s">%s' % (
-	  self.name,self.name,self._accessKeyAttr(),default_html,default_str
+	'<input type="hidden" %stitle="%s" name="%s" %s  value="%s">%s' % (
+	  self.idAttrStr(id_value),
+          (title or self.text).encode(self.charset),
+          self.name,
+          self._accessKeyAttr(),
+          default_html,
+          default_str
+      )
+    )
+
+
+class File(Input):
+  """
+  File upload field
+  <input type="file">
+  """
+  mimeType='application/octet-stream'
+
+  def _validateFormat(self,value):
+    """Binary data is assumed to be valid all the time"""
+    return
+
+  def _encodeValue(self,value):
+    """
+    Return Unicode object or string to be stored in self.value
+    """
+    return value
+
+  def inputHTML(self,default=None,id_value=None,title=None,mimeType=None):
+    return self.inputHTMLTemplate % (
+	'<input type="file" %stitle="%s" name="%s" %s maxlength="%d" size="%d" accept="%s" value="%s">' % (
+	self.idAttrStr(id_value),
+        (title or self.text).encode(self.charset),
+        self.name,
+        self._accessKeyAttr(),
+        self.maxLen,
+        self.size,
+        mimeType or self.mimeType,
+        self._defaultHTML(default)
       )
     )
 
@@ -340,12 +378,18 @@ class Password(Input):
   <input type=password>
   """
 
-  def inputHTML(self,default=None):
+  def inputHTML(self,default=None,id_value=None,title=None):
     return self.inputHTMLTemplate % (
-             '<input id="%s" name="%s" %s maxLen="%d" size="%d" type="password" value="">' % (
-	       self.name,self.name,self._accessKeyAttr(),self.maxLen,self.size
-	     )
-	   )
+      '<input %stitle="%s" name="%s" %s maxlength="%d" size="%d" type="password" value="%s">' % (
+	self.idAttrStr(id_value),
+        (title or self.text).encode(self.charset),
+        self.name,
+        self._accessKeyAttr(),
+        self.maxLen,
+        self.size,
+        default or ''
+      )
+    )
 
   def valueHTML(self):
     """For security reasons only stars are printed"""
@@ -363,7 +407,7 @@ class Radio(Field):
 
   def __init__(
     self,name,text,maxValues=1,required=0,default=None,accessKey='',
-    options=[]
+    options=None
   ):
     """
     pattern and maxLen are determined from __init__ params
@@ -375,6 +419,14 @@ class Radio(Field):
       [('value1','description1),('value2','description2),..]
       for options with different option value and description.
     """
+    self.options = options or []
+    Field.__init__(
+      self,name,text,0,maxValues,'',required,default,accessKey
+    )
+    self.setOptions(options)
+    self.setDefault(default)
+
+  def setOptions(self,options):
     maxLen = 0
     if options:
       optionValues = []
@@ -389,12 +441,11 @@ class Radio(Field):
       pattern   = u'(%s)' % u'|'.join(map(re.escape,optionValues))
     else:
       pattern   = u''
+    self.maxLen = maxLen
     self.options  = options
-    Field.__init__(
-      self,name,text,maxLen,maxValues,pattern,required,default,accessKey
-    )
+    self.setRegex(pattern)
 
-  def inputHTML(self,default=None):
+  def inputHTML(self,default=None,id_value=None,title=None):
     s = []
     default_value = self._defaultValue(default)
     for i in self.options:
@@ -402,22 +453,27 @@ class Radio(Field):
         optionValue,optionText = i
       else:
         optionValue = optionText = i
-      s.append('<input type="radio" id="%s" name="%s" %s value="%s"%s>%s<br>' % (
-          self.name,self.name,
-          self._accessKeyAttr(),optionValue,
+      s.append("""
+        <input
+          type="radio"
+          id="%s"
+          title="%s"
+          name="%s"
+          %s
+          value="%s"
+          %s
+        >%s<br>""" % (
+          self.idAttrStr(id_value),
+          (title or self.text).encode(self.charset),
+          self.name.encode(self.charset),
+          self._accessKeyAttr(),
+          optionValue,
           ' checked'*(optionValue==default_value),
           optionText
 	)
       )
     return self.inputHTMLTemplate % '\n'.join(s)
 
-  def _defaultValue(self,default):
-    """HTML output of default."""
-    if default is None:
-      return self.__dict__.get('default',[])
-    else:
-      return default
-
   def setDefault(self,default):
     """
     Set the default of a default field.
@@ -450,7 +506,7 @@ class Select(Radio):
 
   def __init__(
     self,name,text,maxValues,required=0,default=None,accessKey='',
-    options=[],size=1,ignoreCase=0,multiSelect=0,
+    options=None,size=1,ignoreCase=0,multiSelect=0,
   ):
     """
     Additional parameters:
@@ -470,9 +526,15 @@ class Select(Radio):
       self,name,text,maxValues,required,default,accessKey,options,
     )
 
-  def inputHTML(self,default=None):
-    s = ['<select id="%s" name="%s" %s  size="%d" %s>' % (
-      self.name,self.name,self._accessKeyAttr(),self.size," multiple"*(self.multiSelect>0))]
+  def inputHTML(self,default=None,id_value=None,title=None):
+    s = ['<select %stitle="%s" name="%s" %s  size="%d" %s>' % (
+      self.idAttrStr(id_value),
+      (title or self.text).encode(self.charset),
+      self.name,
+      self._accessKeyAttr(),
+      self.size,
+      " multiple"*(self.multiSelect>0)
+    )]
     default_value = self._defaultValue(default)
     for i in self.options:
       if type(i) is types.TupleType:
@@ -508,18 +570,22 @@ class Checkbox(Field):
     pattern and maxLen are determined by default
     """
     pattern = default
-    maxLen = len(default)
+    maxLen = len(default or '')
     self.checked = checked
     Field.__init__(
       self,name,text,maxLen,maxValues,pattern,required,default,accessKey
     )
 
-  def inputHTML(self,default=None):
+  def inputHTML(self,default=None,id_value=None,title=None,checked=None):
+    if checked is None:
+      checked = self.checked
     return self.inputHTMLTemplate % (
-      '<input type="checkbox" id="%s" name="%s" %s value="%s"%s>' % (
-        self.name,self.name,
+      '<input type="checkbox" %stitle="%s" name="%s" %s value="%s"%s>' % (
+        self.idAttrStr(id_value),
+        (title or self.text).encode(self.charset),
+        self.name,
 	self._accessKeyAttr(),
-	self._defaultValue(default),' checked'*self.checked
+	self._defaultValue(default),' checked'*(checked)
       )
     )
 
@@ -543,12 +609,16 @@ class Keygen(Field):
   def _encodeValue(self,value):
     return value.replace('\n','').replace('\r','')
 
-  def inputHTML(self,challenge):
+  def inputHTML(self,challenge,id_value=None,title=None,):
     return self.inputHTMLTemplate % (
-             '<keygen id="%s" name="%s" %s challenge="%s">' % (
-               self.name,self.name,self._accessKeyAttr(),challenge
-	     )
-	   )
+      '<keygen %stitle="%s" name="%s" %s challenge="%s">' % (
+        self.idAttrStr(id_value),
+        (title or self.text).encode(self.charset),
+        self.name,
+        self._accessKeyAttr(),
+        challenge
+      )
+    )
 
   def valueHTML(self):
     return self.valueHTMLTemplate % ('%d Bytes' % (len(self.value)))
@@ -704,7 +774,7 @@ class Form:
   Class for declaring and processing a whole <form>
   """
 
-  def __init__(self,inf=sys.stdin,env=os.environ):
+  def __init__(self,inf,env):
     """
     Initialize a Form
     inf                 Read from this file object if method is POST.
@@ -719,7 +789,7 @@ class Form:
     # Save the environment vars
     self.env = env
     # input file object
-    self.inf = inf
+    self.inf = inf or sys.stdin
     # Save request method
     self.request_method = env['REQUEST_METHOD']
     self.script_name = env['SCRIPT_NAME']
@@ -731,17 +801,25 @@ class Form:
     # Set the preferred character set
     self.accept_charset = self.http_accept_charset.preferred()
     # Determine query string and content length dependent on request method
-    if self.request_method=='POST':
-      # Parameter von self.inf lesen
-      self.content_length = int(env['CONTENT_LENGTH'])
-      self.query_string = self.inf.read(self.content_length)
-    elif self.request_method=='GET':
-      self.query_string = env.get('QUERY_STRING','')
-      self.content_length = len(self.query_string)
-    else:
-      raise InvalidRequestMethod(self.request_method)
+    self.checkRequestMethod()
     return # Form.__init__()
 
+  def checkRequestMethod(self):
+    """
+    Checks whether the HTTP request method is accepted
+    """
+    if not self.request_method in ['POST','GET']:
+      raise InvalidRequestMethod(self.request_method)
+
+  def getContentType(self):
+    """
+    Determine the HTTP content type of HTTP request
+    """
+    if self.request_method=='POST':
+      return self.env.get('CONTENT_TYPE','application/x-www-form-urlencoded').lower() or None
+    elif self.request_method=='GET':
+      return 'application/x-www-form-urlencoded'
+
   def addField(self,f):
     """
     Add a input field object f to the form.
@@ -778,43 +856,22 @@ class Form:
     ]:
       for v in f.value:
         outf.write(
-          '<input type="hidden" name="%s" value="%s">' % (
-            f.name.encode(f.charset),v.encode(f.charset)
+          '<input type="hidden" name="%s" value="%s">\n\r' % (
+            f.name.encode(f.charset),escapeHTML(v.encode(f.charset))
           )
         )
     return # Form.hiddenInputFields()
 
-  def getInputFields(
-    self,
-    ignoreEmptyFields=0,
-    ignoreUndeclaredFields=0,
-    stripValues=1,
-  ):
-    """
-    Process user's <form> input and store the values in each
-    field instance's content attribute.
+  def _parseFormUrlEncoded(self,maxContentLength,ignoreEmptyFields,ignoreUndeclaredFields,stripValues,unquote):
+      
+    if self.request_method=='POST':
+      query_string = self.inf.read(int(self.env['CONTENT_LENGTH']))
+    elif self.request_method=='GET':
+      query_string = self.env.get('QUERY_STRING','')
 
-    When a processing error occurs FormException (or derivatives)
-    are raised.
+    self.inf.close()
 
-    ignoreEmptyFields=0         Ignore fields with empty input.
-    ignoreUndeclaredFields=0,   Ignore fields with names not declared.
-                                Normally UndeclaredFieldName is raised.
-    stripValues=1,              If true leading and trailing whitespaces
-                                are stripped from all input values.
-    """
-
-    # Calculate maxContentLength
-    maxContentLength = 0
-    for name,f in self.field.items():
-      maxContentLength += f.maxValues*f.maxLen
-
-    # Parse user's input
-    inputlist = self.query_string.split('&')
-
-    # Any input present?
-    if not len(inputlist):
-      return
+    inputlist = query_string.split('&')
 
     contentLength = 0
 
@@ -828,7 +885,7 @@ class Form:
           name,value = param.split('=',1)
         except ValueError:
           raise InvalidFormEncoding(param)
-	name = urllib.unquote_plus(name).strip()
+	name = unquote(name).strip()
         
         if not name in self.declaredFieldNames:
           if ignoreUndeclaredFields:
@@ -836,7 +893,7 @@ class Form:
           else:
             raise UndeclaredFieldName(name)
         
-	value = urllib.unquote_plus(value)
+	value = unquote(value)
         if stripValues:
   	  value = value.strip()
 
@@ -855,6 +912,88 @@ class Form:
           if not name in self.inputFieldNames:
 	    self.inputFieldNames.append(name)
 
+    return #_parseFormUrlEncoded()
+
+
+  def _parseMultipartFormData(self,maxContentLength,ignoreEmptyFields,ignoreUndeclaredFields,stripValues,unquote):
+
+    import cgi
+    ctype, pdict = cgi.parse_header(self.env['CONTENT_TYPE'])
+    parts = cgi.parse_multipart(self.inf,pdict)
+
+    contentLength = 0
+
+    for name in parts.keys():
+
+      if not name in self.declaredFieldNames:
+        if ignoreUndeclaredFields:
+          continue
+        else:
+          raise UndeclaredFieldName(name)
+
+      for value in parts[name]:
+
+#        if stripValues:
+#  	  value = value.strip()
+
+        contentLength += len(value)
+        # Gesamtlaenge der Daten noch zulaessig?
+        if contentLength > maxContentLength:
+          raise ContentLengthExceeded(contentLength,maxContentLength)
+
+        f = self.field[name]
+
+        # input is empty string?
+        if value or (not ignoreEmptyFields):
+	  # Input is stored in field instance
+          f.setValue(value)
+          # Add name of field to list of input keys
+          if not name in self.inputFieldNames:
+	    self.inputFieldNames.append(name)
+
+    return # _parseMultipartFormData()
+
+
+  def getInputFields(
+    self,
+    ignoreEmptyFields=0,
+    ignoreUndeclaredFields=0,
+    stripValues=1,
+    unquotePlus=0,
+  ):
+    """
+    Process user's <form> input and store the values in each
+    field instance's content attribute.
+
+    When a processing error occurs FormException (or derivatives)
+    are raised.
+
+    ignoreEmptyFields=0         Ignore fields with empty input.
+    ignoreUndeclaredFields=0    Ignore fields with names not declared.
+                                Normally UndeclaredFieldName is raised.
+    stripValues=1               If true leading and trailing whitespaces
+                                are stripped from all input values.
+    unquotePlus=0
+       If non-zero urllib.unquote_plus() is used instead of urllib.unquote().
+    """
+
+    unquote = {0:urllib.unquote_plus,1:urllib.unquote_plus}[unquotePlus]
+
+    # Calculate maxContentLength
+    maxContentLength = 0
+    for name,f in self.field.items():
+      maxContentLength += f.maxValues*f.maxLen
+
+    content_type = self.getContentType()
+    if content_type.startswith('application/x-www-form-urlencoded'):
+      # Parse user's input
+      self._parseFormUrlEncoded(maxContentLength,ignoreEmptyFields,ignoreUndeclaredFields,stripValues,unquote)
+    elif content_type.startswith('multipart/form-data'):
+      self._parseMultipartFormData(maxContentLength,ignoreEmptyFields,ignoreUndeclaredFields,stripValues,unquote)
+    else:
+      raise FormException('Invalid content type %s received' % (repr(content_type)))
+
+
     # Are all required parameters present?
     missing_params = []
     for n,f in self.field.items():
