@@ -6,7 +6,7 @@ This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
 
-$Id: httphelper.py,v 1.9 2002/09/26 15:06:39 michael Exp $
+$Id: httphelper.py,v 1.10 2004/06/11 22:53:13 michael Exp $
 """
 
 __version__ = '0.0.4'
@@ -114,11 +114,11 @@ def URLRedirect(outf,url,refreshtime=0,msg='Redirecting...'):
   outf.write("""
 <html>
   <head>
-    <meta http-equiv="refresh" content="%(refreshtime)s; url=%(url)s">
+    <meta http-equiv="refresh" content="%d; url=%s">
   </head>
   <body>
-    %(msg)s <a href="%(url)s">Redirecting...</a>
+    %s <a href="%s">Redirecting...</a>
   </body>
 </html>
-    """ % vars()
+    """ % (refreshtime,url,msg,url)
   )
