@@ -9,10 +9,10 @@ This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
 
-$Id: session.py,v 1.24 2004/03/31 19:27:43 michael Exp $
+$Id: session.py,v 1.26 2004/08/23 19:32:56 michael Exp $
 """
 
-__version__ = '0.3.3'
+__version__ = '0.3.4'
 
 import string,re,random,time,pickle
 
@@ -180,17 +180,23 @@ class WebSession:
         the application trys to access the session ID again.
     crossCheckVars
         List of keys of variables cross-checked for each
-        retrieval of session data in retrieveSession()
+        retrieval of session data in retrieveSession(). If None
+        SESSION_CROSSCHECKVARS is used.
     maxSessionCount
         Maximum number of valid sessions. This affects
         behaviour of retrieveSession() which raises.
         None means unlimited number of sessions.
     """
-    self.sessiondict = dictobj or {}
+    if dictobj is None:
+      self.sessiondict = {}
+    else:
+      self.sessiondict = dictobj
     self.expireDeactivate = expireDeactivate
     self.expireRemove = expireRemove
     self._session_lock = ThreadingLock()
-    self.crossCheckVars = crossCheckVars or SESSION_CROSSCHECKVARS
+    if crossCheckVars is None:
+      crossCheckVars = SESSION_CROSSCHECKVARS
+    self.crossCheckVars = crossCheckVars
     self.maxSessionCount = maxSessionCount
     self.sessionCounter = 0
     self.session_id_re = re.compile('^[a-zA-Z0-9]+$')
